import asyncio
import json
import random
import re
import string
import bs4
import httpx
from .baseservice import BaseService
from .utils.network_utils import fetch_async
from .utils.search_utils import SearchParams

CHARACTERS = string.ascii_lowercase + string.digits

SIZE_MAP = {
    "XS": "spec_id=100100%3A110001%2C100001%3A100101",
    "S": "spec_id=100100%3A110001%2C100001%3A100102",
    "M": "spec_id=100100%3A110001%2C100001%3A100103",
    "L": "spec_id=100100%3A110001%2C100001%3A100104",
    "XL": "spec_id=100100%3A110001%2C100001%3A100105",
    "XXL": "spec_id=100100%3A110001%2C100001%3A100106",
    "FREE / ONESIZE": "100100%3A110001%2C100001%3A100109",
}

BRANDS_MAP = {
    "JUNYA WATANABE": "1623",
    "JUNYA WATANABE MAN": "15429",
    "BLACK COMME des GARCONS": "1624",
    "COMME des GARCONS HOMME": "7319",
    "COMME des GARCONS HOMME DEUX": "7320",
    "COMME des GARCONS SHIRT": "7321",
    "JUNYA WATANABE COMME des GARCONS": "7389",
    "tricot COMME des GARCONS": "7529",
    "COMME des GARCONS HOMME HOMME": "7812",
    "KAPITAL": "1527",
    "nanamica": "6185",
    "noir kei ninomiya": "16960",
    "goa": "542",
    "FULLCOUNT": "5602",
    "WHITESVILLE": "8689",
    "WAREHOUSE": "281",
    "takahiro miyashita the soloist": "14631",
}

CATEGORIES_MAP = {
    "tops": "30",
    "outerwear": "31",
    "pants": "32",
    "shoes": "33",
    "bags": "34",
    "hats": "36",
    "accessories": "38",
    "jewelry": "37",
}


class YJPService(BaseService):

    async def fetch_data_async(self, params: dict) -> httpx.Response:
        self.item_count = params.get("item_count", 50)
        url = self.get_search_params(params)
        res = await fetch_async(url)
        return res

    def fetch_data(self, params: dict) -> httpx.Response:
        self.item_count = params.get("item_count", 50)
        url = self.get_search_params(params)
        res = httpx.get(url)
        return res

    async def parse_response_async(self, response: httpx.Response) -> str:
        soup = bs4.BeautifulSoup(response.text, "lxml")
        results = soup.select(".Product")
        cleaned_items_list = self.get_base_details(results)
        cleaned_items_list_with_details = await self.add_details_async(
            cleaned_items_list
        )
        return json.dumps(cleaned_items_list_with_details)

    def parse_response(self, response: httpx.Response) -> str:
        soup = bs4.BeautifulSoup(response.text, "lxml")
        results = soup.select(".Product")
        cleaned_items_list = self.get_base_details(results)
        cleaned_items_list_with_details = self.add_details(cleaned_items_list)
        return json.dumps(cleaned_items_list_with_details)

    def get_base_details(self, results) -> list:
        cleaned_items_list = []
        for item in results[: self.item_count]:
            temp = {}
            temp["url"] = item.select_one(".Product__image a")["href"]
            temp["id"] = temp["url"].split("/")[-1]
            temp["title"] = item.select_one(".Product__title").text
            price_string = item.select_one(".Product__price").text
            temp["price"] = float(re.sub(r"[^\d.]", "", price_string))
            temp["img"] = ["IMG PLACEHOLDER"]
            temp["size"] = "no size"
            temp["brand"] = "BRAND PLACEHOLDER"
            cleaned_items_list.append(temp)
        return cleaned_items_list

    async def add_details_async(self, items: list) -> list:
        tasks = []
        for item in items:
            url = item["url"]
            task = asyncio.create_task(self.fetch_item_page_async(url))
            tasks.append(task)
        responses = await asyncio.gather(*tasks)
        item_details = [response.text for response in responses]

        for i, details in enumerate(item_details):
            items[i] = {**items[i], **self.parse_item_details(details)}

        return items

    def add_details(self, items: list) -> list:
        for i, item in enumerate(items):
            url = item["url"]
            response = self.fetch_item_page(url)
            details = self.parse_item_details(response.text)
            items[i] = {**items[i], **details}
        return items

    async def fetch_item_page_async(self, url):
        response = await fetch_async(url)
        return response

    def fetch_item_page(self, url):
        response = httpx.get(url)
        return response

    def parse_item_details(self, response_text: str):
        soup = bs4.BeautifulSoup(response_text, "lxml")
        details = {}
        tr_rows = soup.find_all("tr")
        if len(tr_rows) > 1:
            for tr in tr_rows:
                if tr.text.replace("\n", "").replace(" ", "").startswith("サイズ"):
                    details["size"] = tr.td.text.replace("\n", "").replace(" ", "")
                    break
            for tr in tr_rows:
                if (
                    tr.text.replace("\n", "")
                    .replace(" ", "")
                    .startswith("メーカー・ブランド")
                ):
                    details["brand"] = tr.td.text.replace("\n", "").replace(" ", "")
                    break

        images = soup.select(".ProductImage__images img")
        if len(images) > 0:
            details["img"] = [img["src"] for img in images]
        return details

    def get_search_params(self, params: dict) -> str:

        url = "https://auctions.yahoo.co.jp/search/search?&fixed=1&s1=new&n=50"

        if "keyword" in params and params["keyword"] is not "":
            keyword = params["keyword"].replace(" ", "%20")
            url += f"&p={keyword}"
        else:
            url += "&p=sacai"

        size = params.get("size")
        if "size" in params and size is not None:
            if size not in SIZE_MAP:
                raise ValueError(f"Size {size} is not supported")
            size_id = SIZE_MAP[size]
            url += f"&{size_id}"

        page = params.get("page")
        if "page" in params and page is not None:
            url += f"&b={page*50+1}"

        brands = params.get("brand")
        if "brand" in params and brands is not None and len(brands) > 0:
            if brands[0] not in BRANDS_MAP:
                raise ValueError(f"Brand {brands[0]} is not supported")
            brand_id = BRANDS_MAP[brands[0]]
            url += f"&brand_id={brand_id}"

        categories = params.get("category")
        if "category" in params and categories is not None and len(categories) > 0:
            if categories[0] not in CATEGORIES_MAP:
                raise ValueError(f"Category {categories[0]} is not supported")
            category_id = CATEGORIES_MAP[categories[0]]
            url += f"&category_id={category_id}"

        return url



import base64
import uuid
import time
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import serialization
import httpx
from jose import jwt

def get_pop_jwt(url, method="GET"):
    private_key = ec.generate_private_key(ec.SECP256R1())
    public_key = private_key.public_key()

    header = {
        "typ": "dpop+jwt",
        "alg": "ES256",
        "jwk": {
            "crv": "P-256",
            "kty": "EC",
            "x": base64.urlsafe_b64encode(public_key.public_numbers().x.to_bytes(32, 'big')).rstrip(b'=').decode('utf-8'),
            "y": base64.urlsafe_b64encode(public_key.public_numbers().y.to_bytes(32, 'big')).rstrip(b'=').decode('utf-8'),
        },
    }

    payload = {
        "iat": int(time.time()),
        "jti": str(uuid.uuid4()), 
        "htu": url,
        "htm": method,
        "uuid": str(uuid.uuid4()),
    }

    private_key_pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption()
    )

    pop_jwt = jwt.encode(payload, private_key_pem,
                         algorithm='ES256', headers=header)
    
    return pop_jwt

async def fetch_async(url):
    async with httpx.AsyncClient(timeout=15) as client:
        response = await client.get(url)
        return response



import json
import random
import string
import httpx
import bs4
from .baseservice import BaseService
from .utils.search_utils import SearchParams
from .utils.network_utils import fetch_async
import re
import asyncio


CHARACTERS = string.ascii_lowercase + string.digits

SIZE_MAP = {
    "FREE / ONESIZE": 19998,
    "XS": 10001,
    "S": 10003,
    "M": 10004,
    "L": 10005,
    "XL": 10008,
    "XXL": 10009,
}

BRANDS_MAP = {
    "JUNYA WATANABE": "4433",
    "JUNYA WATANABE MAN": "12400",
    "BLACK COMME des GARCONS": "5454",
    "COMME des GARCONS HOMME": "16922",
    "COMME des GARCONS HOMME DEUX": "16918",
    "COMME des GARCONS SHIRT": "16920",
    "JUNYA WATANABE COMME des GARCONS": "5420",
    "tricot COMME des GARCONS": "16921",
    "KAPITAL": "1785",
    "nanamica": "3393",
    "noir kei ninomiya": "12388",
    "goa": "321",
    "FULLCOUNT": "2651",
    "WHITESVILLE": "9493",
    "WAREHOUSE": "4290",
    "takahiro miyashita the soloist": "8964",
}


class FrilService(BaseService):

    async def fetch_data_async(self, params: dict) -> httpx.Response:
        self.item_count = params.get("item_count", 36)
        url = self.get_search_params(params)
        res = await fetch_async(url)
        return res

    def fetch_data(self, params: dict) -> httpx.Response:
        self.item_count = params.get("item_count", 36)
        url = self.get_search_params(params)
        res = httpx.get(url)
        return res

    async def parse_response_async(self, response: httpx.Response) -> str:
        soup = bs4.BeautifulSoup(response.text, "lxml")
        results = soup.select(".item")
        cleaned_items_list = self.get_base_details(results)
        cleaned_items_list_with_details = await self.add_details_async(
            cleaned_items_list
        )
        return json.dumps(cleaned_items_list_with_details)

    def parse_response(self, response: httpx.Response) -> str:
        soup = bs4.BeautifulSoup(response.text, "lxml")
        results = soup.select(".item")
        cleaned_items_list = self.get_base_details(results)
        cleaned_items_list_with_details = self.add_details(cleaned_items_list)
        return json.dumps(cleaned_items_list_with_details)

    def get_base_details(self, results) -> list:
        cleaned_items_list = []
        for item in results[: self.item_count]:
            id = item.select_one(".link_search_image")["href"].split("/")[-1]
            temp = {}
            temp["id"] = id
            temp["title"] = item.select_one(".link_search_image")["title"]
            price_string = item.select_one(".item-box__item-price").text
            temp["price"] = float(re.sub(r"\D", "", price_string))
            temp["url"] = item.select_one(".link_search_image")["href"]
            temp["img"] = ["IMG PLACEHOLDER"]
            temp["size"] = "SIZE PLACEHOLDER"
            temp["brand"] = "BRAND PLACEHOLDER"
            cleaned_items_list.append(temp)
        return cleaned_items_list

    async def add_details_async(self, items: list) -> list:
        tasks = []
        for item in items:
            url = item["url"]
            task = asyncio.create_task(self.fetch_item_page_async(url))
            tasks.append(task)
        responses = await asyncio.gather(*tasks)
        item_details = [response.text for response in responses]

        for i, details in enumerate(item_details):
            items[i] = {**items[i], **self.parse_item_details(details)}

        return items

    def add_details(self, items: list) -> list:
        for i, item in enumerate(items):
            url = item["url"]
            response = self.fetch_item_page(url)
            details = self.parse_item_details(response.text)
            items[i] = {**items[i], **details}
        return items

    async def fetch_item_page_async(self, url):
        response = await fetch_async(url)
        return response

    def fetch_item_page(self, url):
        response = httpx.get(url)
        return response

    def parse_item_details(self, response_text: str):
        soup = bs4.BeautifulSoup(response_text, "lxml")
        details = {}
        tr_rows = soup.find_all("tr")
        if len(tr_rows) > 1:
            details["size"] = tr_rows[1].td.text
            details["brand"] = tr_rows[2].td.text.replace("\n", "")

        sp_slides = soup.find_all("div", class_="sp-slide")
        if len(sp_slides) > 0:
            details["img"] = [sp_slides[0].img["src"]]
        return details

    def get_search_params(self, params: dict) -> str:

        base_url = "https://fril.jp/s?"

        if "keyword" in params:
            url = (
                base_url
                + f"query={params['keyword']}&order=desc&sort=created_at&transaction=selling"
            )

        size = params.get("size")
        if "size" in params and size is not None:
            if size not in SIZE_MAP:
                raise ValueError(f"Size {size} is not supported")
            size_id = SIZE_MAP[size]
            url += f"&size_group_id=3&size_id={size_id}"

        page = params.get("page")
        if "page" in params and page is not None:
            url += f"&page={page}"

        brands = params.get("brand")
        if "brand" in params and brands is not None and len(brands) > 0:
            if brands[0] not in BRANDS_MAP:
                raise ValueError(f"Brand {brands[0]} is not supported")
            brand_id = BRANDS_MAP[brands[0]]
            url += f"&brand_id={brand_id}"

        return url



import asyncio
import json
import random
import string
import httpx
from .baseservice import BaseService
from .utils.network_utils import get_pop_jwt
from .utils.search_utils import SearchParams

CHARACTERS = string.ascii_lowercase + string.digits

SIZE_MAP = {
    "FREE / ONESIZE": "7",
    "XS": "154",
    "S": "2",
    "M": "3",
    "L": "4",
    "XL": "5",
    "XXL": "155",
}
BREANS_MAP = {
    "JUNYA WATANABE": "1623",
    "JUNYA WATANABE MAN": "15429",
    "BLACK COMME des GARCONS": "1624",
    "COMME des GARCONS HOMME": "7319",
    "COMME des GARCONS HOMME DEUX": "7320",
    "COMME des GARCONS SHIRT": "7321",
    "JUNYA WATANABE COMME des GARCONS": "7389",
    "tricot COMME des GARCONS": "7529",
    "COMME des GARCONS HOMME HOMME": "7812",
    "KAPITAL": "1527",
    "nanamica": "6185",
    "noir kei ninomiya": "16960",
    "goa": "542",
    "FULLCOUNT": "5602",
    "WHITESVILLE": "8689",
    "WAREHOUSE": "281",
    "takahiro miyashita the soloist": "14631",
}


class MercariService(BaseService):

    def generate_payload_and_headers(self, params: dict):
        if not isinstance(params, dict):
            raise TypeError("params must be a dict")

        keyword = params.get("keyword")
        size = params.get("size")
        item_count = params.get("item_count")
        page_number = params.get("page")
        if page_number is None:
            page_token = ""
        else:
            page_token = f"v1:{page_number - 1}"

        if size is not None:
            mapped_size = SIZE_MAP.get(size)
        else:
            mapped_size = None

        brands = params.get("brand")
        print("brands raw")
        print(brands)
        if brands is None:
            brands = []
        else:
            brands = [BREANS_MAP.get(brand) for brand in brands]

        url = "https://api.mercari.jp/v2/entities:search"
        searchSessionId = "".join(random.choice(CHARACTERS) for i in range(32))
        payload = {
            "userId": "",
            "pageSize": item_count,
            "pageToken": page_token,
            "searchSessionId": searchSessionId,
            "indexRouting": "INDEX_ROUTING_UNSPECIFIED",
            "thumbnailTypes": [],
            "searchCondition": {
                "keyword": keyword,
                "excludeKeyword": "",
                "sort": "SORT_CREATED_TIME",
                "order": "ORDER_DESC",
                "status": ["STATUS_ON_SALE"],
                "sizeId": [mapped_size],
                "categoryId": [],
                "brandId": brands,
                "sellerId": [],
                "priceMin": 0,
                "priceMax": 0,
                "itemConditionId": [],
                "shippingPayerId": [],
                "shippingFromArea": [],
                "shippingMethod": [],
                "colorId": [],
                "hasCoupon": False,
                "attributes": [],
                "itemTypes": [],
                "skuIds": [],
            },
            "defaultDatasets": ["DATASET_TYPE_MERCARI", "DATASET_TYPE_BEYOND"],
            "serviceFrom": "suruga",
            "userId": "",
            "withItemBrand": True,
            "withItemSize": True,
        }
        headers = {
            "dpop": get_pop_jwt(url, "POST"),
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "x-platform": "web",
        }

        return payload, headers

    async def fetch_data_async(self, params: dict) -> httpx.Response:
        """
        Fetches data from the Mercari API using the provided search parameters.

        Args:
            params (dict): The search parameters.

        Returns:
            httpx.Response: The response from the API.

        Raises:
            TypeError: If params is not a dictionary.
            Exception: If the API request fails.
        """

        url = "https://api.mercari.jp/v2/entities:search"
        payload, headers = self.generate_payload_and_headers(params)

        async with httpx.AsyncClient() as client:
            response = await client.post(url, json=payload, headers=headers)
            return response

    def fetch_data(self, params: dict) -> httpx.Response:
        """
        Fetches data from the Mercari API using the provided search parameters.

        Args:
            params (dict): The search parameters.

        Returns:
            httpx.Response: The response from the API.

        Raises:
            TypeError: If params is not a dictionary.
            Exception: If the API request fails.
        """

        url = "https://api.mercari.jp/v2/entities:search"
        payload, headers = self.generate_payload_and_headers(params)

        with httpx.Client() as client:
            response = client.post(url, json=payload, headers=headers)
            return response

    def parse_response(self, response: httpx.Response) -> str:
        """
        Parses the response from the Mercari API and returns a JSON string of cleaned items.

        Each item is a dictionary with the following keys:
        - 'id': The item's ID.
        - 'title': The item's name.
        - 'price': The item's price.
        - 'size': The item's size, or None if not available.
        - 'url': The URL of the item.
        - 'img': A list of the item's thumbnails.

        Args:
            response (httpx.Response): The response from the API.

        Returns:
            str: A JSON string of cleaned items.
        """

        items = json.loads(response.text)["items"]
        cleaned_items_list = []
        for item in items:
            temp = {}
            temp["id"] = item["id"]
            temp["title"] = item["name"]
            price = float(item.get("price"))
            temp["price"] = price
            if item.get("itemSize") is not None:
                temp["size"] = item["itemSize"].get("name")
            else:
                temp["size"] = None
            temp["url"] = "https://jp.mercari.com/item/" + item["id"]
            thumbnails = item["thumbnails"]
            temp["img"] = []
            for img in thumbnails:
                temp["img"].append(
                    img.replace("c!/w=240,f=webp/thumb", "item/detail/orig")
                )
            print(item)
            if item.get("itemBrand") is not None:
                temp["brand"] = item.get("itemBrand").get("subName")
            else:
                temp["brand"] = "--"
            cleaned_items_list.append(temp)

        item_json = json.dumps(cleaned_items_list)
        return item_json

    async def parse_response_async(self, response: httpx.Response) -> str:
        return self.parse_response(response)

    def get_search_params(self, data: SearchParams) -> str:
        return ""



import asyncio
from pathlib import Path
import pytest
from pytest_mock import mocker
import httpx
import logging

from postalservice import MercariService, YJPService, FrilService
from postalservice.baseservice import BaseService
from postalservice.utils import SearchParams, SearchResults

from typing import Callable, List, Any


@pytest.fixture(scope="module")
def logger():
    # Create a logger
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.INFO)
    handler = logging.StreamHandler()
    formatter = logging.Formatter("TESTS %(levelname)s: %(message)s ")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


@pytest.fixture(scope="module")
def mercari_service():
    return MercariService()


@pytest.fixture(scope="module")
def yjp_service():
    return YJPService()


@pytest.fixture(scope="module")
def fril_service():
    return FrilService()


@pytest.fixture
def mock_response():
    request = httpx.Request("POST", "https://api.mercari.jp/v2/entities:search")

    return httpx.Response(
        status_code=200,
        content=b'{"meta":{"nextPageToken":"v1:0","previousPageToken":"","numFound":"15000"},"items":[{"id":"m43776722534","sellerId":"439593141","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x80\x80\xe3\x83\x88\xe3\x83\xbc\xe3\x83\x88\xe3\x83\x90\xe3\x83\x83\xe3\x82\xb0\xe3\x80\x80\xe3\x83\x89\xe3\x83\x83\xe3\x83\x88\xe6\x9f\x84\xe3\x80\x80\xe3\x82\xb3\xe3\x83\x83\xe3\x83\x88\xe3\x83\xb3\xe3\x80\x80\xe3\x83\x89\xe3\x83\xa1\xe3\x83\x96\xe3\x83\xa9\xe3\x80\x80\xe5\xb8\x8c\xe5\xb0\x91","price":"12600","created":"1701086182","updated":"1729544249","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m43776722534_1.jpg?1701086182"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"4","shippingPayerId":"2","itemSizes":[],"itemBrand":{"id":"563","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3","subName":"COMME des GARCONS"},"itemPromotions":[],"shopName":"","itemSize":null,"shippingMethodId":"14","categoryId":"353","isNoPrice":false,"title":"","isLiked":false},{"id":"m35043879713","sellerId":"103080527","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"COMME des GAR\xc3\x87ONS HOMME \xe3\x83\xb4\xe3\x82\xa3\xe3\x83\xb3\xe3\x83\x86\xe3\x83\xbc\xe3\x82\xb8\xe3\x82\xb8\xe3\x83\xa3\xe3\x82\xb1\xe3\x83\x83\xe3\x83\x88 s\xe3\x82\xb5\xe3\x82\xa4\xe3\x82\xba","price":"22200","created":"1699453480","updated":"1729544205","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m35043879713_1.jpg?1699612989"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"3","shippingPayerId":"2","itemSizes":[{"id":"2","name":"S"}],"itemBrand":{"id":"7319","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x82\xaa\xe3\x83\xa0","subName":"COMME des GARCONS HOMME"},"itemPromotions":[],"shopName":"","itemSize":{"id":"2","name":"S"},"shippingMethodId":"14","categoryId":"314","isNoPrice":false,"title":"","isLiked":false},{"id":"m55515093695","sellerId":"204232876","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"JUNYA WATANABE \xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3 \xe5\xa4\x89\xe5\xbd\xa2\xe3\x82\xab\xe3\x83\xbc\xe3\x83\x87\xe3\x82\xa3\xe3\x82\xac\xe3\x83\xb3 3D \xe3\x82\xa6\xe3\x83\xbc\xe3\x83\xab","price":"9300","created":"1727357243","updated":"1729544199","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m55515093695_1.jpg?1727357243"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"3","shippingPayerId":"2","itemSizes":[{"id":"3","name":"M"}],"itemBrand":{"id":"7389","name":"\xe3\x82\xb8\xe3\x83\xa5\xe3\x83\xb3\xe3\x83\xa4\xe3\x83\xaf\xe3\x82\xbf\xe3\x83\x8a\xe3\x83\x99\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3","subName":"JUNYA WATANABE COMME des GARCONS"},"itemPromotions":[],"shopName":"","itemSize":{"id":"3","name":"M"},"shippingMethodId":"17","categoryId":"1428","isNoPrice":false,"title":"","isLiked":false},{"id":"m32694745382","sellerId":"920151176","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x80\x80\xe3\x82\xb8\xe3\x83\xa5\xe3\x83\xb3\xe3\x83\xa4\xe3\x83\xaf\xe3\x82\xbf\xe3\x83\x8a\xe3\x83\x99\xe3\x80\x80\xe9\x95\xb7\xe8\xa2\x96\xe3\x80\x80\xe3\x82\xb7\xe3\x83\xa3\xe3\x83\x84\xe3\x80\x80XS \xe7\x99\xbd\xe3\x80\x80\xe7\xb6\xbf","price":"13080","created":"1722694682","updated":"1729544174","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m32694745382_1.jpg?1722694682"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"3","shippingPayerId":"2","itemSizes":[{"id":"154","name":"XS(SS)"}],"itemBrand":{"id":"563","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3","subName":"COMME des GARCONS"},"itemPromotions":[],"shopName":"","itemSize":{"id":"154","name":"XS(SS)"},"shippingMethodId":"17","categoryId":"122","isNoPrice":false,"title":"","isLiked":false},{"id":"m61904499198","sellerId":"670933225","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"junya watanabe man levi\'s \xe8\xa7\xa3\xe4\xbd\x93\xe5\x86\x8d\xe6\xa7\x8b\xe7\xaf\x89\xe3\x83\x87\xe3\x83\x8b\xe3\x83\xa0","price":"11100","created":"1724912178","updated":"1729544172","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m61904499198_1.jpg?1724912178"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"2","shippingPayerId":"1","itemSizes":[{"id":"154","name":"XS(SS)"}],"itemBrand":{"id":"7390","name":"\xe3\x82\xb8\xe3\x83\xa5\xe3\x83\xb3\xe3\x83\xa4\xe3\x83\xaf\xe3\x82\xbf\xe3\x83\x8a\xe3\x83\x99\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x83\x87\xe3\x83\x8b\xe3\x83\xa0","subName":"JUNYA WATANABE COMME des GARCONS DENIM"},"itemPromotions":[],"shopName":"","itemSize":{"id":"154","name":"XS(SS)"},"shippingMethodId":"1","categoryId":"10852","isNoPrice":false,"title":"","isLiked":false},{"id":"m32742578858","sellerId":"767473578","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"\xe3\x83\x88\xe3\x83\xaa\xe3\x82\xb3\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x80\x80\xe3\x82\xbb\xe3\x83\x83\xe3\x83\x88\xe3\x82\xa2\xe3\x83\x83\xe3\x83\x97\xe3\x80\x80\xe3\x83\x95\xe3\x83\xaa\xe3\x83\xab\xe3\x80\x80\xe3\x83\x95\xe3\x83\xa9\xe3\x83\xaf\xe3\x83\xbc\xe3\x80\x8000s \xe3\x82\xb9\xe3\x82\xab\xe3\x83\xbc\xe3\x83\x88\xe3\x80\x80\xe3\x82\xb7\xe3\x83\xa3\xe3\x83\x84","price":"16000","created":"1725305125","updated":"1729544144","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m32742578858_1.jpg?1725305125"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"2","shippingPayerId":"2","itemSizes":[{"id":"3","name":"M"}],"itemBrand":{"id":"7529","name":"\xe3\x83\x88\xe3\x83\xaa\xe3\x82\xb3\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3","subName":"tricot COMME des GARCONS"},"itemPromotions":[],"shopName":"","itemSize":{"id":"3","name":"M"},"shippingMethodId":"17","categoryId":"1489","isNoPrice":false,"title":"","isLiked":false},{"id":"m83543244879","sellerId":"687118644","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"AD1989 \xe3\x82\xb3\xe3\x83\xa0 \xe3\x83\x87 \xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3 / \xe3\x82\xa2\xe3\x82\xb7\xe3\x83\xb3\xe3\x83\xa1\xe3\x83\x88\xe3\x83\xaa\xe3\x83\xbc\xe3\x83\xa9\xe3\x83\x83\xe3\x83\x97 \xe3\x82\xb9\xe3\x82\xa6\xe3\x82\xa7\xe3\x83\x83\xe3\x83\x88 \xe3\x83\x91\xe3\x83\xb3\xe3\x83\x84","price":"18600","created":"1727357146","updated":"1729544133","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m83543244879_1.jpg?1727357146"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"3","shippingPayerId":"2","itemSizes":[{"id":"2","name":"S"}],"itemBrand":{"id":"563","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3","subName":"COMME des GARCONS"},"itemPromotions":[],"shopName":"","itemSize":{"id":"2","name":"S"},"shippingMethodId":"14","categoryId":"1476","isNoPrice":false,"title":"","isLiked":false},{"id":"m10991495749","sellerId":"494528507","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"COMME des GARCONS \xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x80\x80\xe3\x83\x8f\xe3\x83\xbc\xe3\x83\x88\xe3\x80\x80\xe3\x82\xb8\xe3\x83\xa3\xe3\x82\xb1\xe3\x83\x83\xe3\x83\x88\xe3\x80\x80\xe3\x83\xa1\xe3\x83\xb3\xe3\x82\xba","price":"19799","created":"1709640294","updated":"1729544107","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m10991495749_1.jpg?1709640294"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"3","shippingPayerId":"2","itemSizes":[{"id":"3","name":"M"}],"itemBrand":{"id":"7319","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x82\xaa\xe3\x83\xa0","subName":"COMME des GARCONS HOMME"},"itemPromotions":[],"shopName":"","itemSize":{"id":"3","name":"M"},"shippingMethodId":"14","categoryId":"335","isNoPrice":false,"title":"","isLiked":false},{"id":"m76146058470","sellerId":"996163716","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"COMME des GARCONS HOMME PLUS 00AW Jacket","price":"37000","created":"1726666284","updated":"1729544012","thumbnails":["https://static.mercdn.net/c!/w=240,f=webp/thumb/photos/m76146058470_1.jpg?1726666284"],"itemType":"ITEM_TYPE_MERCARI","itemConditionId":"4","shippingPayerId":"2","itemSizes":[{"id":"3","name":"M"}],"itemBrand":{"id":"7321","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3\xe3\x82\xaa\xe3\x83\xa0\xe3\x83\x97\xe3\x83\xaa\xe3\x83\xa5\xe3\x82\xb9","subName":"COMME des GARCONS HOMME PLUS"},"itemPromotions":[],"shopName":"","itemSize":{"id":"3","name":"M"},"shippingMethodId":"14","categoryId":"314","isNoPrice":false,"title":"","isLiked":false},{"id":"ZHoogkkHAkVkwfACrChNVW","sellerId":"0","buyerId":"","status":"ITEM_STATUS_ON_SALE","name":"\xe3\x80\x90\xe9\x9b\xb7\xe5\xb8\x82\xe5\xa0\xb4\xef\xbc\x88\xe3\x83\x9d\xe3\x83\xb3\xe3\x82\xb8\xe3\x83\xa3\xe3\x83\xb3\xef\xbc\x89\xe5\x95\x86\xe5\x93\x81\xe9\x9f\x93\xe5\x9b\xbd\xe7\x9b\xb4\xe9\x80\x81\xe3\x80\x91 COMME des GARCONS(\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\xbb\xe3\x83\x87\xe3\x83\xbb\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3) \xe3\x82\xb8\xe3\x83\xa5\xe3\x83\xb3\xe3\x83\xa4 \xe3\x83\xaf\xe3\x82\xbf\xe3\x83\x8a\xe3\x83\x99 \xe3\x83\x9d\xe3\x82\xb1\xe3\x83\x83\xe3\x83\x88 \xe5\x8d\x8a\xe8\xa2\x96","price":"12481","created":"1729543983","updated":"1729543983","thumbnails":["https://assets.mercari-shops-static.com/-/small/plain/P58nbm4QP5ZHnDixnLZ3uB.webp@webp"],"itemType":"ITEM_TYPE_BEYOND","itemConditionId":"2","shippingPayerId":"0","itemSizes":[],"itemBrand":{"id":"563","name":"\xe3\x82\xb3\xe3\x83\xa0\xe3\x83\x87\xe3\x82\xae\xe3\x83\xa3\xe3\x83\xab\xe3\x82\xbd\xe3\x83\xb3","subName":"COMME des GARCONS"},"itemPromotions":[],"shopName":"","itemSize":null,"shippingMethodId":"0","categoryId":"302","isNoPrice":false,"title":"","isLiked":false}],"components":[],"searchCondition":null,"searchConditionId":""}\n',
        request=request,
    )


@pytest.fixture
def mock_item_response(mock_request):
    return


@pytest.mark.parametrize("service_fixture", ["mercari_service"])
def test_fetch_code_200(service_fixture, mocker, request, logger) -> None:
    # Get the service fixture
    service: BaseService = request.getfixturevalue(service_fixture)

    # Mock the response
    mock_response = mocker.Mock()
    mock_response.status_code = 200
    mocker.patch.object(service, "fetch_data", return_value=mock_response)

    res = service.fetch_data()
    logger.info("Fetched data: %s", res)
    assert res.status_code == 200


@pytest.mark.parametrize("service_fixture", ["mercari_service"])
@pytest.mark.asyncio
async def test_fetch_code_200_async(service_fixture, mocker, request, logger):
    # Get the service fixture
    service: BaseService = request.getfixturevalue(service_fixture)

    # Mock the response
    mock_response = mocker.Mock()
    mock_response.status_code = 200
    mocker.patch.object(service, "fetch_data", return_value=mock_response)

    sparams = SearchParams("comme des garcons")
    res = await service.fetch_data_async(sparams.get_dict())
    logger.info("Fetched data: %s", res)
    assert res.status_code == 200


@pytest.mark.parametrize("service_fixture", ["mercari_service"])
def test_parse_results(service_fixture, mocker, request, logger, mock_response):
    # Get the service fixture
    service = request.getfixturevalue(service_fixture)

    # Mock the response
    mocker.patch.object(service, "fetch_data", return_value=mock_response)

    sparams = SearchParams("comme des garcons")
    res = service.fetch_data(sparams.get_dict())
    items = service.parse_response(res)
    searchresults = SearchResults(items)
    logger.info(searchresults)
    assert searchresults.count() > 0



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="postalservice",
    version="0.1.6",
    author="Aapo Montin",
    description="A package for second hand shopping",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
)



from abc import ABC, abstractmethod
import asyncio
import json
import httpx
from postalservice.utils.search_utils import SearchParams, SearchResults

class BaseService(ABC):
    @abstractmethod
    def fetch_data(self, params: dict) -> httpx.Response:

        pass

    @abstractmethod
    async def fetch_data_async(self, params: dict) -> httpx.Response:

        pass

    @abstractmethod
    def parse_response(self, response: str) -> str:

        pass

    @abstractmethod
    async def parse_response_async(self, response: str) -> str:

        pass


    @abstractmethod
    def get_search_params(self, data: SearchParams) -> str:

        pass

    async def get_search_results_async(self, params: dict) -> SearchResults:
        res = await self.fetch_data_async(params)
        items = await self.parse_response_async(res)
        searchresults = SearchResults(items)
        return searchresults
    
    def get_search_results(self, params: dict) -> SearchResults:
        res = self.fetch_data(params)
        items = self.parse_response(res)
        searchresults = SearchResults(items)
        return searchresults
